@@ -296,7 +296,7 @@ MHD_connection_close (struct MHD_Connection *connection,
     connection->daemon->notify_completed (connection->daemon->
 					  notify_completed_cls, connection,
 					  &connection->client_context,
-					  termination_code);    
+					  termination_code);
   connection->client_aware = MHD_NO;
 }
 
@@ -341,7 +341,13 @@ try_ready_normal_body (struct MHD_Connection *connection)
                                 connection->response_write_position));
   if ((ret == 0) &&
       (0 != (connection->daemon->options & MHD_USE_SELECT_INTERNALLY)))
-    abort ();                   /* serious client API violation */
+    mhd_panic (mhd_panic_cls, __FILE__, __LINE__, 
+#if HAVE_MESSAGES
+	       "API violation"
+#else
+	       NULL
+#endif
+	       );
   if (ret == -1)
     {
       /* either error or http 1.0 transfer, close
@@ -494,7 +500,7 @@ add_extra_headers (struct MHD_Connection *connection)
                                             MHD_HTTP_HEADER_CONTENT_LENGTH))
     {
       SPRINTF (buf,
-               "%llu", 
+               "%llu",
 	       (unsigned long long)connection->response->total_size);
       MHD_add_response_header (connection->response,
                                MHD_HTTP_HEADER_CONTENT_LENGTH, buf);
@@ -630,7 +636,7 @@ build_header_response (struct MHD_Connection *connection)
   memcpy (&data[off], "\r\n", 2);
   off += 2;
   if (off != size)
-    abort ();
+    mhd_panic (mhd_panic_cls, __FILE__, __LINE__, NULL);
   connection->write_buffer = data;
   connection->write_buffer_append_offset = size;
   connection->write_buffer_send_offset = 0;
@@ -701,6 +707,29 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
                           fd_set * read_fd_set,
                           fd_set * write_fd_set,
                           fd_set * except_fd_set, int *max_fd)
+{
+  int ret;
+  struct MHD_Pollfd p;
+
+  memset(&p, 0, sizeof(struct MHD_Pollfd));
+  ret = MHD_connection_get_pollfd(connection, &p);
+  if ( (ret == MHD_YES) && (p.fd >= 0) ) {
+    if (0 != (p.events & MHD_POLL_ACTION_IN)) 
+      do_fd_set(p.fd, read_fd_set, max_fd);    
+    if (0 != (p.events & MHD_POLL_ACTION_OUT)) 
+      do_fd_set(p.fd, write_fd_set, max_fd);    
+  }
+  return ret;
+}
+
+/**
+ * Obtain the pollfd for this connection
+ *
+ * @return MHD_YES on success. If return MHD_YES and p->fd < 0, this 
+ *                 connection is not waiting for any read or write events
+ */
+int
+MHD_connection_get_pollfd(struct MHD_Connection *connection, struct MHD_Pollfd *p)
 {
   int fd;
 
@@ -715,6 +744,7 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
       return MHD_NO;
     }
   fd = connection->socket_fd;
+  p->fd = fd;
   if (fd == -1)
     return MHD_YES;
   while (1)
@@ -750,7 +780,7 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
               continue;
             }
           if (MHD_NO == connection->read_closed)
-            do_fd_set (fd, read_fd_set, max_fd);
+            p->events |= MHD_POLL_ACTION_IN;
           break;
         case MHD_CONNECTION_HEADERS_RECEIVED:
           /* we should never get here */
@@ -760,7 +790,7 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
           EXTRA_CHECK (0);
           break;
         case MHD_CONNECTION_CONTINUE_SENDING:
-          do_fd_set (fd, write_fd_set, max_fd);
+          p->events |= MHD_POLL_ACTION_OUT;
           break;
         case MHD_CONNECTION_CONTINUE_SENT:
           if (connection->read_buffer_offset == connection->read_buffer_size)
@@ -789,7 +819,7 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
             }
           if ((connection->read_buffer_offset < connection->read_buffer_size)
               && (MHD_NO == connection->read_closed))
-            do_fd_set (fd, read_fd_set, max_fd);
+            p->events |= MHD_POLL_ACTION_IN;
           break;
         case MHD_CONNECTION_BODY_RECEIVED:
         case MHD_CONNECTION_FOOTER_PART_RECEIVED:
@@ -800,7 +830,7 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
               connection->state = MHD_CONNECTION_CLOSED;
               continue;
             }
-          do_fd_set (fd, read_fd_set, max_fd);
+          p->events |= MHD_POLL_ACTION_IN;
           /* transition to FOOTERS_RECEIVED
              happens in read handler */
           break;
@@ -810,19 +840,19 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
           break;
         case MHD_CONNECTION_HEADERS_SENDING:
           /* headers in buffer, keep writing */
-          do_fd_set (fd, write_fd_set, max_fd);
+          p->events |= MHD_POLL_ACTION_OUT;
           break;
         case MHD_CONNECTION_HEADERS_SENT:
           EXTRA_CHECK (0);
           break;
         case MHD_CONNECTION_NORMAL_BODY_READY:
-          do_fd_set (fd, write_fd_set, max_fd);
+          p->events |= MHD_POLL_ACTION_OUT;
           break;
         case MHD_CONNECTION_NORMAL_BODY_UNREADY:
           /* not ready, no socket action */
           break;
         case MHD_CONNECTION_CHUNKED_BODY_READY:
-          do_fd_set (fd, write_fd_set, max_fd);
+          p->events |= MHD_POLL_ACTION_OUT;
           break;
         case MHD_CONNECTION_CHUNKED_BODY_UNREADY:
           /* not ready, no socket action */
@@ -831,7 +861,7 @@ MHD_connection_get_fdset (struct MHD_Connection *connection,
           EXTRA_CHECK (0);
           break;
         case MHD_CONNECTION_FOOTERS_SENDING:
-          do_fd_set (fd, write_fd_set, max_fd);
+          p->events |= MHD_POLL_ACTION_OUT;
           break;
         case MHD_CONNECTION_FOOTERS_SENT:
           EXTRA_CHECK (0);
@@ -986,8 +1016,8 @@ parse_cookie_header (struct MHD_Connection *connection)
   char old;
   int quotes;
 
-  hdr = MHD_lookup_connection_value (connection, 
-				     MHD_HEADER_KIND, 
+  hdr = MHD_lookup_connection_value (connection,
+				     MHD_HEADER_KIND,
 				     MHD_HTTP_HEADER_COOKIE);
   if (hdr == NULL)
     return MHD_YES;
@@ -1110,6 +1140,7 @@ parse_initial_message_line (struct MHD_Connection *connection, char *line)
   return MHD_YES;
 }
 
+
 /**
  * Call the handler of the application for this
  * connection.  Handles chunking of the upload
@@ -1117,6 +1148,41 @@ parse_initial_message_line (struct MHD_Connection *connection, char *line)
  */
 static void
 call_connection_handler (struct MHD_Connection *connection)
+{
+  size_t processed;
+
+  if (connection->response != NULL)
+    return;                     /* already queued a response */  
+  processed = 0;
+  connection->client_aware = MHD_YES;
+  if (MHD_NO ==
+      connection->daemon->default_handler (connection->daemon->
+					   default_handler_cls,
+					   connection, connection->url,
+					   connection->method,
+					   connection->version,
+					   NULL, &processed,
+					   &connection->client_context))
+    {
+      /* serious internal error, close connection */
+#if HAVE_MESSAGES
+      MHD_DLOG (connection->daemon,
+		"Internal application error, closing connection.\n");
+#endif
+      connection_close_error (connection);
+      return;
+    }
+}
+
+
+
+/**
+ * Call the handler of the application for this
+ * connection.  Handles chunking of the upload
+ * as well as normal uploads.
+ */
+static void
+process_request_body (struct MHD_Connection *connection)
 {
   size_t processed;
   size_t available;
@@ -1259,7 +1325,13 @@ call_connection_handler (struct MHD_Connection *connection)
           return;
         }
       if (processed > used)
-        abort ();               /* fatal client API violation! */
+        mhd_panic (mhd_panic_cls, __FILE__, __LINE__, 
+#if HAVE_MESSAGES
+		   "API violation"
+#else
+		   NULL
+#endif
+		   );
       if (processed != 0)
         instant_retry = MHD_NO; /* client did not process everything */
       used -= processed;
@@ -1913,7 +1985,7 @@ MHD_connection_handle_idle (struct MHD_Connection *connection)
         case MHD_CONNECTION_CONTINUE_SENT:
           if (connection->read_buffer_offset != 0)
             {
-              call_connection_handler (connection);     /* loop call */
+              process_request_body (connection);     /* loop call */
               if (connection->state == MHD_CONNECTION_CLOSED)
                 continue;
             }
