@@ -36,10 +36,18 @@
 #include "gnutls_global.h"
 #endif
 
+#ifdef HAVE_POLL_H
+#include <poll.h>
+#endif
+
 /**
  * Default connection limit.
  */
+#ifndef WINDOWS
 #define MHD_MAX_CONNECTIONS_DEFAULT FD_SETSIZE -4
+#else
+#define MHD_MAX_CONNECTIONS_DEFAULT FD_SETSIZE
+#endif
 
 /**
  * Default memory allowed per connection.
@@ -75,6 +83,29 @@ static void pthread_kill (int, int) {
 }
 #endif  // __SYMBIAN32__
 
+/**
+ * Default implementation of the panic function
+ */
+static void 
+mhd_panic_std(void *cls,
+	      const char *file,
+	      unsigned int line,
+	      const char *reason)
+{
+  abort ();
+}
+
+
+/**
+ * Handler for fatal errors.
+ */
+MHD_PanicCallback mhd_panic;
+
+/**
+ * Closure argument for "mhd_panic".
+ */
+void *mhd_panic_cls;
+
 /**
  * Trace up to and return master daemon. If the supplied daemon
  * is a master, then return the daemon itself.
@@ -423,7 +454,8 @@ MHD_get_fdset (struct MHD_Daemon *daemon,
   if ((daemon == NULL) || (read_fd_set == NULL) || (write_fd_set == NULL)
       || (except_fd_set == NULL) || (max_fd == NULL)
       || (-1 == (fd = daemon->socket_fd)) || (daemon->shutdown == MHD_YES)
-      || ((daemon->options & MHD_USE_THREAD_PER_CONNECTION) != 0))
+      || ((daemon->options & MHD_USE_THREAD_PER_CONNECTION) != 0)
+      || ((daemon->options & MHD_USE_POLL) != 0))
     return MHD_NO;
 
   FD_SET (fd, read_fd_set);
@@ -449,7 +481,7 @@ MHD_get_fdset (struct MHD_Daemon *daemon,
 
 /**
  * Main function of the thread that handles an individual
- * connection.
+ * connection when MHD_USE_THREAD_PER_CONNECTION.
  */
 static void *
 MHD_handle_connection (void *data)
@@ -462,21 +494,18 @@ MHD_handle_connection (void *data)
   int max;
   struct timeval tv;
   unsigned int timeout;
-  unsigned int now;
+  time_t now;
+#ifdef HAVE_POLL_H
+  struct MHD_Pollfd mp;
+  struct pollfd p;
+#endif
 
-  if (con == NULL)
-    abort ();
   timeout = con->daemon->connection_timeout;
-  while ((!con->daemon->shutdown) && (con->socket_fd != -1))
-    {
-      FD_ZERO (&rs);
-      FD_ZERO (&ws);
-      FD_ZERO (&es);
-      max = 0;
-      MHD_connection_get_fdset (con, &rs, &ws, &es, &max);
+  while ((!con->daemon->shutdown) && (con->socket_fd != -1)) {
       now = time (NULL);
       tv.tv_usec = 0;
-      if (timeout > (now - con->last_activity))
+      if ( (timeout > (now - con->last_activity)) ||
+	   (timeout == 0) )
 	{
 	  /* in case we are missing the SIGALRM, keep going after
 	     at most 1s; see http://lists.gnu.org/archive/html/libmicrohttpd/2009-10/msg00013.html */
@@ -486,30 +515,77 @@ MHD_handle_connection (void *data)
 	    {
 	      /* do not block (we're waiting for our callback to succeed) */
 	      tv.tv_sec = 0;
-	    }	
+	    }
 	}
       else
 	{
 	  tv.tv_sec = 0;
 	}
-      num_ready = SELECT (max + 1, &rs, &ws, &es, &tv);
-      if (num_ready < 0)
-        {
+#ifdef HAVE_POLL_H
+      if (0 == (con->daemon->options & MHD_USE_POLL)) {
+#else
+      {
+#endif
+	/* use select */
+        FD_ZERO (&rs);
+        FD_ZERO (&ws);
+        FD_ZERO (&es);
+        max = 0;
+        MHD_connection_get_fdset (con, &rs, &ws, &es, &max);
+        num_ready = SELECT (max + 1, &rs, &ws, &es, &tv);
+        if (num_ready < 0) {
+            if (errno == EINTR)
+              continue;
+#if HAVE_MESSAGES
+            MHD_DLOG (con->daemon, "Error during select (%d): `%s'\n", max,
+                      STRERROR (errno));
+#endif
+            break;
+        }
+        /* call appropriate connection handler if necessary */
+        if ((con->socket_fd != -1) && (FD_ISSET (con->socket_fd, &rs)))
+          con->read_handler (con);
+        if ((con->socket_fd != -1) && (FD_ISSET (con->socket_fd, &ws)))
+          con->write_handler (con);
+        if (con->socket_fd != -1)
+          con->idle_handler (con);
+      }
+#ifdef HAVE_POLL_H
+      else
+      {
+        /* use poll */
+        memset(&mp, 0, sizeof (struct MHD_Pollfd));
+        MHD_connection_get_pollfd(con, &mp);
+	memset(&p, 0, sizeof (struct pollfd));
+        p.fd = mp.fd;
+        if (mp.events & MHD_POLL_ACTION_IN) 
+          p.events |= POLLIN;        
+        if (mp.events & MHD_POLL_ACTION_OUT) 
+          p.events |= POLLOUT;
+	/* in case we are missing the SIGALRM, keep going after
+	   at most 1s */
+	if (poll (&p, 1, 1000) < 0) {
           if (errno == EINTR)
             continue;
 #if HAVE_MESSAGES
-          MHD_DLOG (con->daemon, "Error during select (%d): `%s'\n", max,
+          MHD_DLOG (con->daemon, "Error during poll: `%s'\n", 
                     STRERROR (errno));
 #endif
           break;
         }
-      /* call appropriate connection handler if necessary */
-      if ((con->socket_fd != -1) && (FD_ISSET (con->socket_fd, &rs)))
-        con->read_handler (con);
-      if ((con->socket_fd != -1) && (FD_ISSET (con->socket_fd, &ws)))
-        con->write_handler (con);
-      if (con->socket_fd != -1)
-        con->idle_handler (con);
+        if ( (con->socket_fd != -1) && 
+	     (0 != (p.revents & POLLIN)) ) 
+          con->read_handler (con);        
+        if ( (con->socket_fd != -1) && 
+	     (0 != (p.revents & POLLOUT)) )
+          con->write_handler (con);        
+        if (con->socket_fd != -1)
+          con->idle_handler (con);
+	if ( (con->socket_fd != -1) &&
+	     (0 != (p.revents & (POLLERR | POLLHUP))) )
+          MHD_connection_close (con, MHD_REQUEST_TERMINATED_WITH_ERROR);      
+      }
+#endif
     }
   if (con->socket_fd != -1)
     {
@@ -579,20 +655,17 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
 #endif
   struct sockaddr *addr = (struct sockaddr *) &addrstorage;
   socklen_t addrlen;
-  int s, res_thread_create;
+  int s;
+  int res_thread_create;
 #if OSX
   static int on = 1;
 #endif
 
-#if HAVE_INET6
-  if (sizeof (struct sockaddr) > sizeof (struct sockaddr_in6))
-    abort ();                   /* fatal, serious error */
-#endif
   addrlen = sizeof (addrstorage);
   memset (addr, 0, sizeof (addrstorage));
 
   s = ACCEPT (daemon->socket_fd, addr, &addrlen);
-  if ((s < 0) || (addrlen <= 0))
+  if ((s == -1) || (addrlen <= 0))
     {
 #if HAVE_MESSAGES
       /* This could be a common occurance with multiple worker threads */
@@ -607,17 +680,20 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
         }
       return MHD_NO;
     }
-  if (s >= FD_SETSIZE)
+#ifndef WINDOWS
+  if ( (s >= FD_SETSIZE) &&
+       (0 == (daemon->options & MHD_USE_POLL)) )
     {
 #if HAVE_MESSAGES
       MHD_DLOG (daemon,
 		"Socket descriptor larger than FD_SETSIZE: %d > %d\n",
 		s,
 		FD_SETSIZE);
-#endif     
+#endif
       CLOSE (s);
       return MHD_NO;
     }
+#endif
 
 
 #if HAVE_MESSAGES
@@ -718,7 +794,19 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
                     "Failed to setup TLS credentials: unknown credential type %d\n",
                     connection->daemon->cred_type);
 #endif
-          abort ();
+          SHUTDOWN (s, SHUT_RDWR);
+          CLOSE (s);
+          MHD_ip_limit_del (daemon, addr, addrlen);
+          free (connection->addr);
+          free (connection);
+          mhd_panic (mhd_panic_cls, __FILE__, __LINE__, 
+#if HAVE_MESSAGES
+		     "Unknown credential type"
+#else
+		     NULL
+#endif
+		     );
+ 	  return MHD_NO;
         }
       MHD__gnutls_transport_set_ptr (connection->tls_session,
                                      (MHD_gnutls_transport_ptr_t) connection);
@@ -740,7 +828,7 @@ MHD_accept_connection (struct MHD_Daemon *daemon)
         {
 #if HAVE_MESSAGES
           MHD_DLOG (daemon, "Failed to create a thread: %s\n",
-                    STRERROR (errno));
+                    STRERROR (res_thread_create));
 #endif
           SHUTDOWN (s, SHUT_RDWR);
           CLOSE (s);
@@ -790,7 +878,7 @@ MHD_cleanup_connections (struct MHD_Daemon *daemon)
 		  MHD_DLOG (daemon, "Failed to join a thread: %s\n",
 			    STRERROR (rc));
 #endif
-		  abort();		
+		  abort();
 		}
             }
           MHD_destroy_response (pos->response);
@@ -877,8 +965,6 @@ MHD_select (struct MHD_Daemon *daemon, int may_block)
 
   timeout.tv_sec = 0;
   timeout.tv_usec = 0;
-  if (daemon == NULL)
-    abort ();
   if (daemon->shutdown == MHD_YES)
     return MHD_NO;
   FD_ZERO (&rs);
@@ -905,11 +991,11 @@ MHD_select (struct MHD_Daemon *daemon, int may_block)
         return MHD_NO;
       FD_SET (max, &rs);
     }
-  
+
   /* in case we are missing the SIGALRM, keep going after
      at most 1s; see http://lists.gnu.org/archive/html/libmicrohttpd/2009-10/msg00013.html */
   timeout.tv_usec = 0;
-  timeout.tv_sec = 1; 
+  timeout.tv_sec = 1;
   if (may_block == MHD_NO)
     {
       timeout.tv_usec = 0;
@@ -969,6 +1055,42 @@ MHD_select (struct MHD_Daemon *daemon, int may_block)
   return MHD_YES;
 }
 
+/**
+ * Poll for new connection. Used only with THREAD_PER_CONNECTION
+ */
+static int
+MHD_poll (struct MHD_Daemon *daemon)
+{
+#ifdef HAVE_POLL_H
+  struct pollfd p;
+
+  if (0 == (daemon->options & MHD_USE_THREAD_PER_CONNECTION)) 
+    return MHD_NO;
+  p.fd = daemon->socket_fd;
+  p.events = POLLIN;
+  p.revents = 0;
+
+  if (poll(&p, 1, 1000) < 0) {
+    if (errno == EINTR)
+      return MHD_YES;
+#if HAVE_MESSAGES
+    MHD_DLOG (daemon, "poll failed: %s\n", STRERROR (errno));
+#endif
+    return MHD_NO;
+  }
+  /* handle shutdown cases */
+  if (daemon->shutdown == MHD_YES) 
+    return MHD_NO;  
+  if (daemon->socket_fd < 0) 
+    return MHD_YES; 
+  if (0 != (p.revents & POLLIN)) 
+    MHD_accept_connection (daemon);
+  return MHD_YES;
+#else
+  return MHD_NO;
+#endif
+}
+
 /**
  * Run webserver operations (without blocking unless
  * in client callbacks).  This method should be called
@@ -1001,7 +1123,10 @@ MHD_select_thread (void *cls)
   struct MHD_Daemon *daemon = cls;
   while (daemon->shutdown == MHD_NO)
     {
-      MHD_select (daemon, MHD_YES);
+      if ((daemon->options & MHD_USE_POLL) == 0) 
+	MHD_select (daemon, MHD_YES);
+      else 
+	MHD_poll(daemon);      
       MHD_cleanup_connections (daemon);
     }
   return NULL;
@@ -1037,6 +1162,237 @@ MHD_start_daemon (unsigned int options,
 
 typedef void (*VfprintfFunctionPointerType)(void *, const char *, va_list);
 
+
+/**
+ * Parse a list of options given as varargs.
+ * 
+ * @param daemon the daemon to initialize
+ * @param ap the options
+ * @return MHD_YES on success, MHD_NO on error
+ */
+static int
+parse_options_va (struct MHD_Daemon *daemon,
+		  const struct sockaddr **servaddr,
+		  va_list ap);
+
+
+/**
+ * Parse a list of options given as varargs.
+ * 
+ * @param daemon the daemon to initialize
+ * @param ... the options
+ * @return MHD_YES on success, MHD_NO on error
+ */
+static int
+parse_options (struct MHD_Daemon *daemon,
+	       const struct sockaddr **servaddr,
+	       ...)
+{
+  va_list ap;
+  int ret;
+
+  va_start (ap, servaddr);
+  ret = parse_options_va (daemon, servaddr, ap);
+  va_end (ap);
+  return ret;
+}
+
+
+/**
+ * Parse a list of options given as varargs.
+ * 
+ * @param daemon the daemon to initialize
+ * @param ap the options
+ * @return MHD_YES on success, MHD_NO on error
+ */
+static int
+parse_options_va (struct MHD_Daemon *daemon,
+		  const struct sockaddr **servaddr,
+		  va_list ap)
+{
+  enum MHD_OPTION opt;
+  struct MHD_OptionItem *oa;
+  unsigned int i;
+  
+  while (MHD_OPTION_END != (opt = va_arg (ap, enum MHD_OPTION)))
+    {
+      switch (opt)
+        {
+        case MHD_OPTION_CONNECTION_MEMORY_LIMIT:
+          daemon->pool_size = va_arg (ap, size_t);
+          break;
+        case MHD_OPTION_CONNECTION_LIMIT:
+          daemon->max_connections = va_arg (ap, unsigned int);
+          break;
+        case MHD_OPTION_CONNECTION_TIMEOUT:
+          daemon->connection_timeout = va_arg (ap, unsigned int);
+          break;
+        case MHD_OPTION_NOTIFY_COMPLETED:
+          daemon->notify_completed =
+            va_arg (ap, MHD_RequestCompletedCallback);
+          daemon->notify_completed_cls = va_arg (ap, void *);
+          break;
+        case MHD_OPTION_PER_IP_CONNECTION_LIMIT:
+          daemon->per_ip_connection_limit = va_arg (ap, unsigned int);
+          break;
+        case MHD_OPTION_SOCK_ADDR:
+          *servaddr = va_arg (ap, const struct sockaddr *);
+          break;
+        case MHD_OPTION_URI_LOG_CALLBACK:
+          daemon->uri_log_callback =
+            va_arg (ap, LogCallback);
+          daemon->uri_log_callback_cls = va_arg (ap, void *);
+          break;
+        case MHD_OPTION_THREAD_POOL_SIZE:
+          daemon->worker_pool_size = va_arg (ap, unsigned int);
+          break;
+#if HTTPS_SUPPORT
+        case MHD_OPTION_PROTOCOL_VERSION:
+	  if (daemon->options & MHD_USE_SSL)
+	    _set_priority (&daemon->priority_cache->protocol,
+			   va_arg (ap, const int *));
+#if HAVE_MESSAGES
+	  else
+	    FPRINTF (stderr,
+		     "MHD HTTPS option %d passed to MHD but MHD_USE_SSL not set\n",
+		     opt);	  
+#endif
+          break;
+        case MHD_OPTION_HTTPS_MEM_KEY:
+	  if (daemon->options & MHD_USE_SSL)
+	    daemon->https_mem_key = va_arg (ap, const char *);
+#if HAVE_MESSAGES
+	  else
+	    FPRINTF (stderr,
+		     "MHD HTTPS option %d passed to MHD but MHD_USE_SSL not set\n",
+		     opt);	  
+#endif
+          break;
+        case MHD_OPTION_HTTPS_MEM_CERT:
+	  if (daemon->options & MHD_USE_SSL)
+	    daemon->https_mem_cert = va_arg (ap, const char *);
+#if HAVE_MESSAGES
+	  else
+	    FPRINTF (stderr,
+		     "MHD HTTPS option %d passed to MHD but MHD_USE_SSL not set\n",
+		     opt);	  
+#endif
+          break;
+        case MHD_OPTION_CIPHER_ALGORITHM:
+	  if (daemon->options & MHD_USE_SSL)
+	    _set_priority (&daemon->priority_cache->cipher,
+			   va_arg (ap, const int *));
+#if HAVE_MESSAGES
+	  else
+	    FPRINTF (stderr,
+		     "MHD HTTPS option %d passed to MHD but MHD_USE_SSL not set\n",
+		     opt);	  
+#endif	  
+          break;
+#endif
+        case MHD_OPTION_EXTERNAL_LOGGER:
+#if HAVE_MESSAGES
+          daemon->custom_error_log =
+            va_arg (ap, VfprintfFunctionPointerType);
+          daemon->custom_error_log_cls = va_arg (ap, void *);
+#else
+          va_arg (ap, VfprintfFunctionPointerType);
+          va_arg (ap, void *);
+#endif
+          break;
+	case MHD_OPTION_ARRAY:
+	  oa = va_arg (ap, struct MHD_OptionItem*);
+	  i = 0;
+	  while (MHD_OPTION_END != (opt = oa[i].option))
+	    {
+	      switch (opt)
+		{
+		  /* all options taking 'size_t' */
+		case MHD_OPTION_CONNECTION_MEMORY_LIMIT:
+		  if (MHD_YES != parse_options (daemon,
+						servaddr,
+						opt,
+						(size_t) oa[i].value,
+						MHD_OPTION_END))
+		    return MHD_NO;
+		  break;
+		  /* all options taking 'unsigned int' */
+		case MHD_OPTION_CONNECTION_LIMIT:
+		case MHD_OPTION_CONNECTION_TIMEOUT:
+		case MHD_OPTION_PER_IP_CONNECTION_LIMIT:
+		case MHD_OPTION_THREAD_POOL_SIZE:
+		  if (MHD_YES != parse_options (daemon,
+						servaddr,
+						opt,
+						(unsigned int) oa[i].value,
+						MHD_OPTION_END))
+		    return MHD_NO;
+		  break;
+		  /* all options taking 'int' or 'enum' */
+		case MHD_OPTION_CRED_TYPE:
+		  if (MHD_YES != parse_options (daemon,
+						servaddr,
+						opt,
+						(int) oa[i].value,
+						MHD_OPTION_END))
+		    return MHD_NO;
+		  break;
+		  /* all options taking one pointer */
+		case MHD_OPTION_SOCK_ADDR:
+		case MHD_OPTION_HTTPS_MEM_KEY:
+		case MHD_OPTION_HTTPS_MEM_CERT:
+		case MHD_OPTION_PROTOCOL_VERSION:
+		case MHD_OPTION_CIPHER_ALGORITHM:
+		case MHD_OPTION_ARRAY:
+		  if (MHD_YES != parse_options (daemon,
+						servaddr,
+						opt,
+						oa[i].ptr_value,
+						MHD_OPTION_END))
+		    return MHD_NO;
+		  break;
+		  /* all options taking two pointers */
+		case MHD_OPTION_NOTIFY_COMPLETED:
+		case MHD_OPTION_URI_LOG_CALLBACK:
+		case MHD_OPTION_EXTERNAL_LOGGER:
+		  if (MHD_YES != parse_options (daemon,
+						servaddr,
+						opt,
+						(void *) oa[i].value,
+						oa[i].ptr_value,
+						MHD_OPTION_END))
+		    return MHD_NO;
+		  break;
+		  
+		default:
+		  return MHD_NO;
+		}
+	      i++;
+	    }
+	  break;
+        default:
+#if HAVE_MESSAGES
+          if ((opt >= MHD_OPTION_HTTPS_MEM_KEY) &&
+              (opt <= MHD_OPTION_CIPHER_ALGORITHM))
+            {
+              FPRINTF (stderr,
+                       "MHD HTTPS option %d passed to MHD compiled without HTTPS support\n",
+                       opt);
+            }
+          else
+            {
+              FPRINTF (stderr,
+                       "Invalid option %d! (Did you terminate the list with MHD_OPTION_END?)\n",
+                       opt);
+            }
+#endif
+	  return MHD_NO;
+        }
+    }  
+  return MHD_YES;
+}
+
+
 /**
  * Start a webserver on the given port.
  *
@@ -1064,8 +1420,8 @@ MHD_start_daemon_va (unsigned int options,
 #endif
   const struct sockaddr *servaddr = NULL;
   socklen_t addrlen;
-  enum MHD_OPTION opt;
   unsigned int i;
+  int res_thread_create;
 
   if ((port == 0) || (dh == NULL))
     return NULL;
@@ -1096,15 +1452,14 @@ MHD_start_daemon_va (unsigned int options,
 #if HAVE_MESSAGES
 	  MHD_DLOG (retVal, "Failed to aquire gnutls mutex\n");
 #endif
-	  abort();
+          mhd_panic (mhd_panic_cls, __FILE__, __LINE__, NULL);
 	}
-      MHD__gnutls_global_init ();
       if (0 != pthread_mutex_unlock (&MHD_gnutls_init_mutex))
 	{
 #if HAVE_MESSAGES
 	  MHD_DLOG (retVal, "Failed to release gnutls mutex\n");
 #endif
-	  abort();
+	  mhd_panic (mhd_panic_cls, __FILE__, __LINE__, NULL);
 	}
       /* set default priorities */
       MHD_tls_set_default_priority (&retVal->priority_cache, "", NULL);
@@ -1112,84 +1467,23 @@ MHD_start_daemon_va (unsigned int options,
     }
 #endif
 
-  while (MHD_OPTION_END != (opt = va_arg (ap, enum MHD_OPTION)))
+  if (MHD_YES != parse_options_va (retVal, &servaddr, ap))
     {
-      switch (opt)
-        {
-        case MHD_OPTION_CONNECTION_MEMORY_LIMIT:
-          retVal->pool_size = va_arg (ap, size_t);
-          break;
-        case MHD_OPTION_CONNECTION_LIMIT:
-          retVal->max_connections = va_arg (ap, unsigned int);
-          break;
-        case MHD_OPTION_CONNECTION_TIMEOUT:
-          retVal->connection_timeout = va_arg (ap, unsigned int);
-          break;
-        case MHD_OPTION_NOTIFY_COMPLETED:
-          retVal->notify_completed =
-            va_arg (ap, MHD_RequestCompletedCallback);
-          retVal->notify_completed_cls = va_arg (ap, void *);
-          break;
-        case MHD_OPTION_PER_IP_CONNECTION_LIMIT:
-          retVal->per_ip_connection_limit = va_arg (ap, unsigned int);
-          break;
-        case MHD_OPTION_SOCK_ADDR:
-          servaddr = va_arg (ap, struct sockaddr *);
-          break;
-        case MHD_OPTION_URI_LOG_CALLBACK:
-          retVal->uri_log_callback =
-            va_arg (ap, LogCallback);
-          retVal->uri_log_callback_cls = va_arg (ap, void *);
-          break;
-        case MHD_OPTION_THREAD_POOL_SIZE:
-          retVal->worker_pool_size = va_arg (ap, unsigned int);
-          break;
-#if HTTPS_SUPPORT
-        case MHD_OPTION_PROTOCOL_VERSION:
-          _set_priority (&retVal->priority_cache->protocol,
-                         va_arg (ap, const int *));
-          break;
-        case MHD_OPTION_HTTPS_MEM_KEY:
-          retVal->https_mem_key = va_arg (ap, const char *);
-          break;
-        case MHD_OPTION_HTTPS_MEM_CERT:
-          retVal->https_mem_cert = va_arg (ap, const char *);
-          break;
-        case MHD_OPTION_CIPHER_ALGORITHM:
-          _set_priority (&retVal->priority_cache->cipher,
-                         va_arg (ap, const int *));
-          break;
-#endif
-        case MHD_OPTION_EXTERNAL_LOGGER:
-#if HAVE_MESSAGES
-          retVal->custom_error_log =
-            va_arg (ap, VfprintfFunctionPointerType);
-          retVal->custom_error_log_cls = va_arg (ap, void *);
-#else
-          va_arg (ap, VfprintfFunctionPointerType);
-          va_arg (ap, void *);
-#endif
-          break;
-        default:
-#if HAVE_MESSAGES
-          if ((opt >= MHD_OPTION_HTTPS_MEM_KEY) &&
-              (opt <= MHD_OPTION_CIPHER_ALGORITHM))
-            {
-              FPRINTF (stderr,
-                       "MHD HTTPS option %d passed to MHD compiled without HTTPS support\n",
-                       opt);
-            }
-          else
-            {
-              FPRINTF (stderr,
-                       "Invalid option %d! (Did you terminate the list with MHD_OPTION_END?)\n",
-                       opt);
-            }
-#endif
-          abort ();
-        }
+      free (retVal);
+      return NULL;
     }
 
+  /* poll support currently only works with MHD_USE_THREAD_PER_CONNECTION */
+  if ( (0 != (options & MHD_USE_POLL)) && 
+       (0 == (options & MHD_USE_THREAD_PER_CONNECTION)) ) {
+#if HAVE_MESSAGES
+      fprintf (stderr,
+               "MHD poll support only works with MHD_USE_THREAD_PER_CONNECTION\n");
+#endif
+      free (retVal);
+      return NULL;
+  }
+
   /* Thread pooling currently works only with internal select thread model */
   if ((0 == (options & MHD_USE_SELECT_INTERNALLY))
       && (retVal->worker_pool_size > 0))
@@ -1221,12 +1515,13 @@ MHD_start_daemon_va (unsigned int options,
 #if HAVE_MESSAGES
       fprintf (stderr, "AF_INET6 not supported\n");
 #endif
+      free (retVal);
       return NULL;
     }
 #endif
   else
-    socket_fd = SOCKET (PF_INET, SOCK_STREAM, 0);  
-  if (socket_fd < 0)
+    socket_fd = SOCKET (PF_INET, SOCK_STREAM, 0);
+  if (socket_fd == -1)
     {
 #if HAVE_MESSAGES
       if ((options & MHD_USE_DEBUG) != 0)
@@ -1235,7 +1530,9 @@ MHD_start_daemon_va (unsigned int options,
       free (retVal);
       return NULL;
     }
-  if (socket_fd >= FD_SETSIZE)
+#ifndef WINDOWS
+  if ( (socket_fd >= FD_SETSIZE) &&
+       (0 == (options & MHD_USE_POLL)) )
     {
 #if HAVE_MESSAGES
       if ((options & MHD_USE_DEBUG) != 0)
@@ -1243,10 +1540,12 @@ MHD_start_daemon_va (unsigned int options,
                  "Socket descriptor larger than FD_SETSIZE: %d > %d\n",
                  socket_fd,
                  FD_SETSIZE);
-#endif     
+#endif
       CLOSE (socket_fd);
+      free (retVal);
       return NULL;
     }
+#endif
   if ((SETSOCKOPT (socket_fd,
                    SOL_SOCKET,
                    SO_REUSEADDR,
@@ -1284,7 +1583,7 @@ MHD_start_daemon_va (unsigned int options,
         }
     }
   retVal->socket_fd = socket_fd;
-  if (BIND (socket_fd, servaddr, addrlen) < 0)
+  if (BIND (socket_fd, servaddr, addrlen) == -1)
     {
 #if HAVE_MESSAGES
       if ((options & MHD_USE_DEBUG) != 0)
@@ -1332,15 +1631,15 @@ MHD_start_daemon_va (unsigned int options,
       return NULL;
     }
 #endif
-  if (((0 != (options & MHD_USE_THREAD_PER_CONNECTION)) ||
-       ((0 != (options & MHD_USE_SELECT_INTERNALLY))
-        && (0 == retVal->worker_pool_size)))
-        && (0 !=
-          pthread_create (&retVal->pid, NULL, &MHD_select_thread, retVal)))
+  if ( ( (0 != (options & MHD_USE_THREAD_PER_CONNECTION)) ||
+	 ( (0 != (options & MHD_USE_SELECT_INTERNALLY)) &&
+	   (0 == retVal->worker_pool_size)) ) && 
+       (0 != (res_thread_create =
+	      pthread_create (&retVal->pid, NULL, &MHD_select_thread, retVal))))
     {
 #if HAVE_MESSAGES
       MHD_DLOG (retVal,
-                "Failed to create listen thread: %s\n", STRERROR (errno));
+                "Failed to create listen thread: %s\n", STRERROR (res_thread_create));
 #endif
       pthread_mutex_destroy (&retVal->per_ip_connection_mutex);
       free (retVal);
@@ -1413,11 +1712,12 @@ MHD_start_daemon_va (unsigned int options,
             ++d->max_connections;
 
           /* Spawn the worker thread */
-          if (0 != pthread_create (&d->pid, NULL, &MHD_select_thread, d))
+          if (0 != (res_thread_create = pthread_create (&d->pid, NULL, &MHD_select_thread, d)))
             {
 #if HAVE_MESSAGES
               MHD_DLOG (retVal,
-                        "Failed to create pool thread: %d\n", STRERROR (errno));
+                        "Failed to create pool thread: %s\n", 
+			STRERROR (res_thread_create));
 #endif
               /* Free memory for this worker; cleanup below handles
                * all previously-created workers. */
@@ -1522,7 +1822,7 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
 	  MHD_DLOG (daemon, "Failed to join a thread: %s\n",
 		    STRERROR (rc));
 #endif
-	  abort();		
+	  abort();
 	}
       MHD_close_connections (&daemon->worker_pool[i]);
     }
@@ -1568,7 +1868,6 @@ MHD_stop_daemon (struct MHD_Daemon *daemon)
 #endif
 	  abort();
 	}
-      MHD__gnutls_global_deinit ();
       if (0 != pthread_mutex_unlock (&MHD_gnutls_init_mutex))
 	{
 #if HAVE_MESSAGES
@@ -1605,6 +1904,27 @@ MHD_get_daemon_info (struct MHD_Daemon *daemon,
     };
 }
 
+/**
+ * Sets the global error handler to a different implementation.  "cb"
+ * will only be called in the case of typically fatal, serious
+ * internal consistency issues.  These issues should only arise in the
+ * case of serious memory corruption or similar problems with the
+ * architecture.  While "cb" is allowed to return and MHD will then
+ * try to continue, this is never safe.
+ *
+ * The default implementation that is used if no panic function is set
+ * simply calls "abort".  Alternative implementations might call
+ * "exit" or other similar functions.
+ *
+ * @param cb new error handler
+ * @param cls passed to error handler
+ */
+void MHD_set_panic_func (MHD_PanicCallback cb, void *cls)
+{
+  mhd_panic = cb;
+  mhd_panic_cls = cls;
+}
+
 /**
  * Obtain the version of this library
  *
@@ -1642,6 +1962,9 @@ sigalrmHandler (int sig)
  */
 void ATTRIBUTE_CONSTRUCTOR MHD_init ()
 {
+  mhd_panic = &mhd_panic_std;
+  mhd_panic_cls = NULL;
+
 #ifndef WINDOWS
   /* make sure SIGALRM does not kill us */
   memset (&sig, 0, sizeof (struct sigaction));
@@ -1650,9 +1973,10 @@ void ATTRIBUTE_CONSTRUCTOR MHD_init ()
   sig.sa_handler = &sigalrmHandler;
   sigaction (SIGALRM, &sig, &old);
 #else
-  plibc_init ("CRISP", "libmicrohttpd");
+  plibc_init ("GNU", "libmicrohttpd");
 #endif
 #if HTTPS_SUPPORT
+  MHD__gnutls_global_init ();
   if (0 != pthread_mutex_init(&MHD_gnutls_init_mutex, NULL))
     abort();
 #endif
@@ -1661,8 +1985,9 @@ void ATTRIBUTE_CONSTRUCTOR MHD_init ()
 void ATTRIBUTE_DESTRUCTOR MHD_fini ()
 {
 #if HTTPS_SUPPORT
+  MHD__gnutls_global_deinit ();
   if (0 != pthread_mutex_destroy(&MHD_gnutls_init_mutex))
-    abort ();
+    mhd_panic (mhd_panic_cls, __FILE__, __LINE__, NULL);
 #endif
 #ifndef WINDOWS
   sigaction (SIGALRM, &old, &sig);
