@@ -27,7 +27,11 @@
 
 #include "reason_phrase.h"
 
-static const char *invalid_hundred[] = { };
+#ifndef NULL
+#define NULL (void*)0
+#endif  // !NULL
+
+static const char *invalid_hundred[] = { NULL };
 
 static const char *one_hundred[] = {
   "Continue",
