@@ -81,7 +81,7 @@ extern "C"
 /**
  * Current version of the library.
  */
-#define MHD_VERSION 0x00040400
+#define MHD_VERSION 0x00040600
 
 /**
  * MHD-internal return code for "YES".
@@ -292,7 +292,14 @@ enum MHD_FLAG
    * recommended to turn this ON if you are testing clients against
    * MHD, and OFF in production.
    */
-  MHD_USE_PEDANTIC_CHECKS = 32
+  MHD_USE_PEDANTIC_CHECKS = 32,
+
+  /**
+   * Use poll instead of select. This allows sockets with fd >= FD_SETSIZE.
+   * This option only works in conjunction with MHD_USE_THREAD_PER_CONNECTION
+   * (at this point).
+   */
+  MHD_USE_POLL = 64
 };
 
 /**
@@ -446,9 +453,58 @@ enum MHD_OPTION
    * (MHD_start_daemon returns NULL for an unsupported thread
    * model).
    */
-  MHD_OPTION_THREAD_POOL_SIZE = 14
+  MHD_OPTION_THREAD_POOL_SIZE = 14,
+
+  /**
+   * Additional options given in an array of "struct MHD_OptionItem".
+   * The array must be terminated with an entry '{MHD_OPTION_END, 0, NULL}'.
+   * An example for code using MHD_OPTION_ARRAY is:
+   * <code>
+   * struct MHD_OptionItem ops[] = {
+   * { MHD_OPTION_CONNECTION_LIMIT, 100, NULL },
+   * { MHD_OPTION_CONNECTION_TIMEOUT, 10, NULL },
+   * { MHD_OPTION_END, 0, NULL }
+   * };
+   * d = MHD_start_daemon(0, 8080, NULL, NULL, dh, NULL,
+   *                      MHD_OPTION_ARRAY, ops,
+   *                      MHD_OPTION_END);
+   * </code>
+   * For options that expect a single pointer argument, the
+   * second member of the struct MHD_OptionItem is ignored.
+   * For options that expect two pointer arguments, the first
+   * argument must be cast to 'intptr_t'.
+   */
+  MHD_OPTION_ARRAY = 15
 };
 
+
+/**
+ * Entry in an MHD_OPTION_ARRAY.
+ */
+struct MHD_OptionItem
+{
+  /**
+   * Which option is being given.  Use MHD_OPTION_END
+   * to terminate the array.
+   */
+  enum MHD_OPTION option;
+
+  /**
+   * Option value (for integer arguments, and for options requiring
+   * two pointer arguments); should be 0 for options that take no
+   * arguments or only a single pointer argument.
+   */
+  intptr_t value;
+
+  /**
+   * Pointer option value (use NULL for options taking no arguments
+   * or only an integer option).
+   */
+  void *ptr_value;
+
+};
+
+
 /**
  * The MHD_ValueKind specifies the source of
  * the key-value pairs in the HTTP protocol.
@@ -577,7 +633,7 @@ enum MHD_ConnectionInfoType
 
   /**
    * Obtain IP address of the client.
-   * Takes no extra arguments.   
+   * Takes no extra arguments.
    */
   MHD_CONNECTION_INFO_CLIENT_ADDRESS
 };
@@ -636,6 +692,18 @@ struct MHD_Response;
  */
 struct MHD_PostProcessor;
 
+/**
+ * Callback for serious error condition. The default action is to abort().
+ * @param cls user specified value
+ * @param file where the error occured
+ * @param line where the error occured
+ * @param reason error detail, may be NULL
+ */
+typedef void (*MHD_PanicCallback) (void *cls,
+                                   const char *file,
+                                   unsigned int line,
+                                   const char *reason);
+
 /**
  * Allow or deny a client to connect.
  *
@@ -758,8 +826,8 @@ typedef int
  *  requests using the same TCP connection).
  */
 typedef int
-  (*MHD_ContentReaderCallback) (void *cls, 
-				uint64_t pos, 
+  (*MHD_ContentReaderCallback) (void *cls,
+				uint64_t pos,
 				char *buf,
 				int max);
 
@@ -952,6 +1020,23 @@ MHD_set_connection_value (struct MHD_Connection *connection,
                           enum MHD_ValueKind kind,
                           const char *key, const char *value);
 
+/**
+ * Sets the global error handler to a different implementation.  "cb"
+ * will only be called in the case of typically fatal, serious
+ * internal consistency issues.  These issues should only arise in the
+ * case of serious memory corruption or similar problems with the
+ * architecture.  While "cb" is allowed to return and MHD will then
+ * try to continue, this is never safe.
+ *
+ * The default implementation that is used if no panic function is set
+ * simply calls "abort".  Alternative implementations might call
+ * "exit" or other similar functions.
+ *
+ * @param cb new error handler
+ * @param cls passed to error handler
+ */
+void MHD_set_panic_func (MHD_PanicCallback cb, void *cls);
+
 /**
  * Get a particular header value.  If multiple
  * values match the kind, return any one of them.
@@ -1212,7 +1297,7 @@ const union MHD_DaemonInfo *MHD_get_daemon_info (struct MHD_Daemon *daemon,
 
 /**
  * Obtain the version of this library
- * 
+ *
  * @return static version string, e.g. "0.4.1"
  */
 const char* MHD_get_version(void);
